from pNbody import *
from numarray import *


def Diff(f,x):
  '''
  First derivative of f(x)
  '''
  
  fp = zeros(len(x),x.type())
    
  fp[0 ]  = (f[ 1]-f[ 0])/(x[ 1]-x[ 0]) 
  fp[-1]  = (f[-1]-f[-2])/(x[-1]-x[-2])
  
  
  f1 = f[2:]
  f2 = f[:-2]
  x1 = x[2:]
  x2 = x[:-2]  
  
  fp[1:-1] = (f1-f2)/(x1-x2)
  
  return fp


def Kappa(R,dPhi,d2Phi):
    
  Kappa2 = where((R==0),0.0,d2Phi + dPhi*3/R)
  return sqrt(Kappa2)


def Omega(R,dPhi):

  Omega2 = where((R==0),0,dPhi/R)
  return sqrt(Omega2)

def Nu(z,Phi):
  
  nr,nz = Phi.shape
  
  odd = fmod(nz,2)
  
  if odd:
    idx1 = (nz-1)/2
  else:
    idx1 = (nz/2)-1
    idx2 = (nz/2)

  
  nu2 = array([],Float)
  
  for i in range(nr):
    
    d1 = Diff(Phi[i,:],z)
    d2 = Diff(d1,z)
        
    if odd:
      nu2 = concatenate((nu2,d2[idx1]))
    else:
      nu2 = concatenate((nu2,0.5*(d2[idx1]+d2[idx2])))
    
  return where(nu2>0,sqrt(nu2),0 )   
 

def Vcirc2(R,dPhi):

  return R*dPhi
    
def Vcirc(R,dPhi):

  Vcirc2 = R*dPhi
  return sqrt(Vcirc2)
  

def QToomre(G,R,SigmaR,Kappa,Sdens):
  
  Q = where((Sdens==0),0,SigmaR*Kappa/(3.36*G*Sdens))
  return Q





def get_1d_Sigma_From_Rho_Phi(rho,phi,r,dr):

  dphi= libgrid.get_First_Derivative(phi,r)

  integrant = rho*dphi
  res = array([],Float)
  nr = len(dr)
  

  for i in range(nr):
    
    ia = i
    ib = nr
    
    I =  libgrid.get_Integral(integrant,dr,ia=ia,ib=ib)
    res = concatenate((res,I))


  sigma = where(rho>0,res/rho,0)
  sigma = sqrt(sigma)  

  return sigma


def get_2d_Sigma_From_Rho_Phi(m,Phi,z,dz):

  res = array([],Float) 
  nr = Phi.shape[0]
  nz = Phi.shape[1]

  for i in range(nr):

    dzPhi  = libgrid.get_First_Derivative(Phi[i,:],z)
    integrant = m[i,:]*dzPhi

    for j in range(nz):

      if (j < (nz-1)/2) :
  	ia = (nz-1) - j
  	ib = nz
      else:
  	ia = j
  	ib = nz
  	  
      I =  libgrid.get_Integral(integrant,dz,ia=ia,ib=ib)
      res = concatenate((res,I))    

  res.shape = (nr,nz)
  
  sigma_z2 = where(m>0,res/m,0)
  sigma_z  = where(sigma_z2>0,sqrt(sigma_z2),0)
  
  
  return sigma_z













